import requests
import datetime
from tabulate import tabulate

# Set the base URLs for the API
BASE_URL = "YOUR_TENANT_URL" # replace with your tenant URL i.e "https://mycompany.signalsresearch.revvitycloud.com/"
BASE_API_URL = f"{BASE_URL}api/rest/v1.0/"

HEADERS = {
    "accept": "application/vnd.api+json",
    "x-api-key": "YOUR_API_KEY", # replace with API key generated from System Settings
    "Content-Type": "application/vnd.api+json"
}

def fetch_all_users():
    """Fetches all users from the API, handling pagination automatically."""
    users_data = []
    included_roles = []
    PAGE_LIMIT = 100
    user_endpoint = f"{BASE_API_URL}users?enabled=true&page[offset]=0&page[limit]={PAGE_LIMIT}"
    
    while True:
        response = requests.get(user_endpoint, headers=HEADERS)
        if response.status_code != 200:
            print("Error fetching data:", response.status_code)
            return None
        
        data = response.json()
        users_data.extend(data.get("data", []))
        
        # Store role data separately to resolve user roles later
        if "included" in data:
            included_roles.extend(data["included"])  # Ensure we gather all roles
        
        # Check for pagination and continue fetching if needed
        next_link = data.get("links", {}).get("next")
        if not next_link:
            break  # No more pages to fetch
        
        user_endpoint = next_link
    
    return {"data": users_data, "included": included_roles}

def fetch_license_usage():
    """Fetches license usage details from the API."""
    response = requests.get(BASE_API_URL + "users/licenses", headers=HEADERS)
    if response.status_code != 200:
        print("Error fetching license data:", response.status_code)
        return None
    
    data = response.json()
    licenses = data.get("data", {}).get("attributes", {}).get("licenses", [])
    
    license_summary = []
    for license in licenses:
        total = license["totalLicense"]
        used = license["totalLicenseConsumed"]
        remaining = total - used
        license_summary.append({
            "License Name": license["name"],
            "Total Licenses": total,
            "Used Licenses": used,
            "Remaining Licenses": remaining
        })
    
    return license_summary

def fetch_entity_creation_total():
    """Fetches number of entities were created for the entities we are interested in in the last 30 days from the tags search API."""
    
    # Get the current time in UTC
    now = datetime.datetime.now(datetime.timezone.utc)
    # Calculate the time 30 days ago
    thirty_days_ago = now - datetime.timedelta(days=30)
    
    # Format both dates to match the Signals expected date format
    from_date = thirty_days_ago.strftime("%Y-%m-%dT%H:%M:%S.000Z")
    to_date = now.strftime("%Y-%m-%dT%H:%M:%S.000Z")

    payload = {
        "query": {
            "$and": [
                {
                    "$in": {
                        "field": "type",
                          "values": [
                            "experiment",
                            "sample",
                            "task"
                          ]
                    }
                },
                {
                    "$range": {
                        "field": "createdAt",
                        "as": "date",
                        "from": from_date,
                        "to": to_date
                    }
                },
                {
                    "$match": {
                      "field": "isTemplate",
                      "value": False
                    }
                }
            ]
        },
        "field": "type"
    }

    response = requests.post(BASE_API_URL + "entities/search/terms?source=SN", headers=HEADERS, json=payload)
    
    if response.status_code != 200:
        print("Error fetching entity data:", response.status_code)
        return None
    
    data = response.json()

    return [{"Entity Type": entry["attributes"]["term"], "Total Created": entry["attributes"]["count"]} for entry in data["data"]]

def parse_users(data):
    """Parses user data and extracts relevant attributes."""
    users = []
    role_mapping = {r["id"]: r["attributes"]["name"] for r in data.get("included", []) if r["type"] == "role"}
    now = datetime.datetime.utcnow()
    
    for user in data["data"]:
        attributes = user["attributes"]
        last_login = attributes.get("lastLoginAt")
        last_login_date = datetime.datetime.strptime(last_login, "%Y-%m-%dT%H:%M:%S.%fZ") if last_login else None
        days_since_login = (now - last_login_date).days if last_login_date else None
        
        roles = [role_mapping.get(role["id"], "Unknown") for role in user["relationships"].get("roles", {}).get("data", [])]
        licenses = [license["name"] for license in attributes.get("licenses", [])]
        
        users.append({
            "User ID": attributes.get("userId"),
            "First Name": attributes.get("firstName"),
            "Last Name": attributes.get("lastName"),
            "Email": attributes.get("email"),
            "Last Login": last_login_date.strftime("%Y-%m-%d") if last_login_date else "Never",
            "Days Since Login": days_since_login if last_login_date else "Never",
            "Licenses": ", ".join(licenses),
            "Roles": ", ".join(roles)
        })
    
    return users

def categorize_users(users):
    """Categorizes users based on their last login activity."""
    never_logged_in = [u for u in users if u["Last Login"] == "Never"]
    over_30_days = [u for u in users if isinstance(u["Days Since Login"], int) and u["Days Since Login"] > 30]
    active_users = [u for u in users if isinstance(u["Days Since Login"], int) and u["Days Since Login"] <= 30]
    
    return never_logged_in, over_30_days, active_users

def display_users(users, category):
    """Displays user details for a selected category."""
    print(f"\n{category} Users:")
    print(tabulate(users, headers="keys", tablefmt="pretty"))

def display_summary(never_logged_in, over_30_days, active_users):
    """Displays a summary of user login activity."""
    print("\nUser Summary Report:")
    print(f"Total Users: {len(never_logged_in) + len(over_30_days) + len(active_users)}")
    print(f"Users who never logged in: {len(never_logged_in)}")
    print(f"Users who haven't logged in for 30+ days: {len(over_30_days)}")
    print(f"Active users (logged in within 30 days): {len(active_users)}")

def display_license_summary(license_summary):
    """Displays the license usage summary in tabular format."""
    print("\nLicense Usage Summary:")
    print(tabulate(license_summary, headers="keys", tablefmt="pretty"))

def display_entity_summary(entity_creation_summary):
    """Displays a summary of entity creation in tabular format."""
    print("\nEntity Creation Summary Report (past 30 days):")
    print(tabulate(entity_creation_summary, headers="keys", tablefmt="pretty"))

def main():
    users_data = fetch_all_users()
    if not users_data:
        return
    
    users = parse_users(users_data)
    never_logged_in, over_30_days, active_users = categorize_users(users)
    
    while True:
        display_summary(never_logged_in, over_30_days, active_users)
    
        license_summary = fetch_license_usage()
        entity_creation_summary = fetch_entity_creation_total()
        if license_summary and entity_creation_summary:
            display_license_summary(license_summary)
            display_entity_summary(entity_creation_summary)
        choice = input("\nView user details for: (1) Active, (2) Inactive (30+ days), (3) Never Logged In, (q) Quit: ")
        if choice == "1":
            display_users(active_users, "Active")
            input("Press Enter to return")
        elif choice == "2":
            display_users(over_30_days, "Inactive (30+ days)")
            input("Press Enter to return")
        elif choice == "3":
            display_users(never_logged_in, "Never Logged In")
            input("Press Enter to return")
        elif choice.lower() == "q":
            break
        else:
            print("Invalid choice, please try again.")
            continue

if __name__ == "__main__":
    main()
